package main

import (
	"fmt"
	"image/color"
	"log"
	"math"
	"os"

	"github.com/hajimehoshi/ebiten"
	"github.com/hajimehoshi/ebiten/ebitenutil"
	"github.com/hajimehoshi/ebiten/inpututil"
	"github.com/golang/freetype/truetype"
	"golang.org/x/image/font"
	"golang.org/x/image/font/opentype"
)

const (
	screenWidth  = 800
	screenHeight = 600
	tileSize     = 64
	tileRadius   = tileSize / 2
	tileHeight   = 100
	cameraSpeed  = 4
)

var (
	tiles      [][]Tile
	cameraX, cameraY float64
	fontFace   font.Face
)

type TileType int

const (
	Field TileType = iota
	Mountain
	Forest
)

type Tile struct {
	X, Y   int
	Type   TileType
	River  bool
}

func init() {
	fontBytes, err := os.ReadFile("font.ttf")
	if err != nil {
		log.Fatal(err)
	}
	tt, err := truetype.Parse(fontBytes)
	if err != nil {
		log.Fatal(err)
	}
	fontFace = truetype.NewFace(tt, &truetype.Options{
		Size:    16,
		DPI:     72,
		Hinting: font.HintingFull,
	})

	tiles = make([][]Tile, 5)
	for i := range tiles {
		tiles[i] = make([]Tile, 5)
		for j := range tiles[i] {
			tiles[i][j] = Tile{
				X:    j,
				Y:    i,
				Type: Field,
			}
		}
	}
}

func main() {
	if err := ebiten.Run(update, screenWidth, screenHeight, 1, "Hexagonal Tiles Example"); err != nil {
		log.Fatal(err)
	}
}

func update(screen *ebiten.Image) error {
	if ebiten.IsKeyPressed(ebiten.KeyW) {
		cameraY -= cameraSpeed
	}
	if ebiten.IsKeyPressed(ebiten.KeyS) {
		cameraY += cameraSpeed
	}
	if ebiten.IsKeyPressed(ebiten.KeyA) {
		cameraX -= cameraSpeed
	}
	if ebiten.IsKeyPressed(ebiten.KeyD) {
		cameraX += cameraSpeed
	}

	if inpututil.IsMouseButtonJustReleased(ebiten.MouseButtonLeft) {
		x, y := ebiten.CursorPosition()
		tileX := int(math.Floor((float64(x) - cameraX) / tileSize))
		tileY := int(math.Floor((float64(y) - cameraY) / tileSize))

		if tileX >= 0 && tileX < len(tiles[0]) && tileY >= 0 && tileY < len(tiles) {
			tiles[tileY][tileX].Type = (tiles[tileY][tileX].Type + 1) % 3
		}
	}

	for i := range tiles {
		for j := range tiles[i] {
			drawTile(screen, tiles[i][j])
		}
	}

	text.Draw(screen, fmt.Sprintf("Camera: (%.2f, %.2f)", cameraX, cameraY), fontFace, 10, 20, color.White)
	x, y := ebiten.CursorPosition()
	text.Draw(screen, fmt.Sprintf("Cursor: (%d, %d)", x, y), fontFace, 10, 40, color.White)
	text.Draw(screen, fmt.Sprintf("FPS: %.2f", ebiten.CurrentFPS()), fontFace, 10, 60, color.White)

	return nil
}

func drawTile(screen *ebiten.Image, tile Tile) {
	x := float64(tile.X*tileSize) + cameraX
	y := float64(tile.Y*tileSize) + cameraY

	color := color.White
	switch tile.Type {
	case Mountain:
		color = color.RGBA{255, 0, 0, 255}
	case Forest:
		color = color.RGBA{0, 255, 0, 255}
	}

	pts := hexagonPoints(x, y)
	ebitenutil.DrawPolygon(screen, pts, color)

	if tile.River {
		ebitenutil.DrawLine(screen, pts[0].Position.X, pts[0].Position.Y, pts[3].Position.X, pts[3].Position.Y, color.RGBA{0, 0, 255, 255})
		ebitenutil.DrawLine(screen, pts[3].Position.X, pts[3].Position.Y, pts[6].Position.X, pts[6].Position.Y, color.RGBA{0, 0, 255, 255})
	}

	text.Draw(screen, fmt.Sprintf("(%d, %d)", tile.X, tile.Y), fontFace, int(x)+tileRadius-20, int(y)+tileHeight+20, color.White)
}

func hexagonPoints(x, y float64) []ebiten.Vertex {
	pts := make([]ebiten.Vertex, 7)
	for i := 0; i < 6; i++ {
		angle := 2 * math.Pi * float64(i) / 6
		dx := tileRadius * math.Cos(angle)
		dy := tileRadius * math.Sin(angle)
		pts[i] = ebiten.Vertex{
			Position: ebiten.Vector{
				X: x + dx,
				Y: y + dy,
			},
			ColorR: 1,
			ColorG: 1,
			ColorB: 1,
			ColorA: 1,
		}
	}
	pts[6] = pts[0]
	return pts
}